from . import download
download.downloading()
from gc import collect
import bpy
import math
import random
import os
from . import translate

bl_info = {
	"name": "Fade Assets",
	"category": "3D View",
	"author": "Thatimster and Wayward Art co",
	"blender": (4, 2, 0),
	"location": "View3D > Properties",
	"description": "Add Toon Nature Assets to any scene.",
	"warning": "",
	"wiki_url": "",
	"tracker_url": "",
	"version": (1,4,3)
}


class UPDATE_OT_fog(bpy.types.Operator):
	bl_idname = 'update.fog'
	bl_description = 'Update Fog Group'
	bl_category = 'Node'
	bl_label = 'Copy To Selected'
	bl_options = {'REGISTER', 'UNDO'}

	update_type: bpy.props.StringProperty()

	def execute(self, context):
		selected_obj = context.scene.fade_fog_obj
		if not selected_obj or not selected_obj.active_material or not selected_obj.active_material.node_tree or not check_fog_exists(selected_obj.active_material.node_tree.nodes):
			context.scene.fade_fog_obj = None
			return {'FINISHED'}

		settings = {}
		for n in selected_obj.active_material.node_tree.nodes:
			if n.type == "GROUP" and n.node_tree.name.startswith("Fog"):
				settings["Start"] = n.inputs["Start"].default_value
				settings["End"] = n.inputs["End"].default_value
				settings["Fade"] = n.inputs["Fade"].default_value
				settings["Alpha"] = n.inputs["Alpha"].default_value
				settings["Color"] = n.inputs["Color"].default_value
				break
		if self.update_type == "selected":
			iter_objs = context.selected_objects
		else:
			iter_objs = [
				o for o in context.scene.objects if o.active_material and o.active_material.node_tree]
		for s in iter_objs:
			for m in [slot.material for slot in s.material_slots if slot.material and slot.material != selected_obj.active_material]:
				if m.node_tree:
					existing = self.check_fog_exists(m.node_tree.nodes)
					if existing == None:
						add_fog_to_obj(context, [s])
						existing = self.check_fog_exists(
							s.active_material.node_tree.nodes)
					for k in settings.keys():
						existing.inputs[k].default_value = settings[k]

		return {'FINISHED'}

	def check_fog_exists(self, nodes):
		found = None
		for n in nodes:
			if n.type == "GROUP" and n.node_tree.name.startswith("Fog"):
				found = n
				break
		return found


class ADD_OT_fog(bpy.types.Operator):
	bl_idname = 'add.fog'
	bl_description = 'Add Fog Group'
	bl_category = 'Node'
	bl_label = 'Add Fog'
	bl_options = {'REGISTER', 'UNDO'}

	@classmethod
	def poll(cls, context):
		return context.active_object is not None

	def execute(self, context):
		to_add = [ob for ob in context.selected_objects if ob !=
				  context.scene.fade_fog_obj and not check_fog_exists(ob.active_material.node_tree.nodes)]
		add_fog_to_obj(context, to_add)
		return {'FINISHED'}

	def check_fog_exists(self, nodes):
		found = False
		for n in nodes:
			if n.type == "GROUP" and n.node_tree.name.startswith("Fog"):
				found = True
				break
		return found

# add fog function


def add_fog_to_obj(context, selection):
	mats = set()
	for o in selection:
		for m in [slot.material for slot in o.material_slots if slot.material]:
			mats.add(m)
	existing = [ng for ng in bpy.data.node_groups if ng.name.startswith("Fog")]
	mat_ref = {'fog': ('Fog')}
	select_mat_name = list(mat_ref.keys())[0]
	select_ng = mat_ref[select_mat_name]

	if not existing:
		file_loc = os.path.join(os.path.dirname(
			__file__), "data", "data.blend")
		with bpy.data.libraries.load(file_loc) as (data_from, data_to):
			data_to.node_groups = [select_ng]

	for m in mats:
		current_tree = m.node_tree
		if existing:
			target_node_tree = existing[0]
		else:
			target_node_tree = bpy.data.node_groups[select_ng]
		current_nodes = current_tree.nodes
		new_n = current_nodes.new(type="ShaderNodeGroup")
		new_n.node_tree = target_node_tree

		mat_out = find_output(current_tree)
		if mat_out:
			frm_n = mat_out.inputs[0].links[0].from_node
			frm_sock_idx = list(frm_n.outputs).index(
				mat_out.inputs[0].links[0].from_socket)
			current_tree.links.new(
				frm_n.outputs[frm_sock_idx], new_n.inputs["Base Material"])
			current_tree.links.new(new_n.outputs[0], mat_out.inputs[0])

			for sock in frm_n.inputs:
				if sock.name.lower() == "alpha" or (frm_n.type == "MIX_SHADER" and sock.name.lower() == "fac"):
					if sock.links:
						src = sock.links[0].from_node
						socket = list(src.outputs).index(
							sock.links[0].from_socket)
						current_tree.links.new(
							src.outputs[socket], new_n.inputs["Alpha"])
						break
			mat_out.location.x += 200
			new_n.location.x = mat_out.location.x - 200
			new_n.location.y = mat_out.location.y


class ADD_OT_fadesky(bpy.types.Operator):
	bl_idname = 'add.sky'
	bl_description = 'Add Sky Group'
	bl_category = 'Node'
	bl_label = 'Add Sky'
	bl_options = {'REGISTER', 'UNDO'}

	def execute(self, context):
		scn = context.scene
		prop = scn.fade_choice

		if scn.fade_sky_shadeless:
			prop = scn.fade_choice + " Shadeless"
		exists = self.check_world_exists(prop)
		if exists == None:
			file_loc = os.path.join(os.path.dirname(
				__file__), "data", "data.blend")
			worlds = set([w.name for w in bpy.data.worlds])
			with bpy.data.libraries.load(file_loc) as (data_from, data_to):
				data_to.worlds = [prop]
			new_world = list(
				set([w.name for w in bpy.data.worlds]).difference(worlds))[0]
			context.scene.world = bpy.data.worlds[new_world]
		else:
			context.scene.world = bpy.data.worlds[prop]
		return {'FINISHED'}

	def check_world_exists(self, name):
		for w in bpy.data.worlds:
			if w.name == name:
				return w
		return None


class ADD_OT_fademodel(bpy.types.Operator):
	bl_idname = 'add.fademodel'
	bl_description = 'add model'
	bl_category = 'Fade'
	bl_label = 'Add Model'
	bl_options = {'REGISTER', 'UNDO'}

	def execute(self, context):
		prop = context.scene.fade_choice
		self.append_collection(context, prop)
		return {"FINISHED"}

	def append_collection(self, context, collection_name):
		if context.scene.fade_category in ["Weather", "Clouds", "Bushes", "Foliage", "Stones", "Grass"]:
			use_particles = context.scene.fade_particles
		else:
			use_particles = False

		file_loc = os.path.join(os.path.dirname(
			__file__), "data", "data.blend")
		if collection_name not in bpy.data.collections:
			with bpy.data.libraries.load(file_loc) as (data_from, data_to):
				data_to.collections = [collection_name]

		target_coll = bpy.data.collections[collection_name]
		total_objs = []
		root_refs = []
		for ob in target_coll.objects:
			obname = ob.name.split(".")[0]
			total_objs.append(obname)
			if ob.parent == None:
				root_refs.append(obname)
			# for c in ob.children:
			#     cname = c.name.split(".")[0]
			#     total_objs.append(cname)
		target_coll = None
		bpy.data.collections.remove(bpy.data.collections[collection_name])

		before = set([obj.name for obj in bpy.data.objects])
		with bpy.data.libraries.load(file_loc) as (data_from, data_to):
			data_to.objects = total_objs
		diff = set([obj.name for obj in bpy.data.objects]).difference(before)
		if use_particles:
			current_coll = bpy.data.collections.new(
				collection_name+"_particles")
			context.scene.collection.children.link(current_coll)
		else:
			current_coll = context.view_layer.active_layer_collection.collection

		positions = []
		root_objs = []
		for ob in list(diff):
			for root in root_refs:
				if ob.startswith(root):
					positions.append(list(bpy.data.objects[ob].location))
					root_objs.append(ob)
					break
			current_coll.objects.link(bpy.data.objects[ob])

		avg = [sum(x)/len(x) for x in zip(*positions)]
		if use_particles:
			to_move = [context.object.location[i] - avg[i] for i in range(3)]
		else:
			to_move = [context.scene.cursor.location[i] - avg[i]
					   for i in range(3)]
		for obj in root_objs:
			cur = bpy.data.objects[obj]
			cur.location.x += to_move[0]
			cur.location.y += to_move[1]
			if use_particles:
				cur.location.z += (to_move[2] +
								   context.scene.fade_particles_assetoffset)
				cur.rotation_euler = (0.0, math.pi / 2, 0.0)
			else:
				cur.location.z += to_move[2]
			cur.scale *= context.scene.fade_scale
			cur.select_set(True)

		if context.scene.fade_fog:
			self.add_fog(context, context.scene.fade_fog_obj != None, [
				context.scene.objects[ob] for ob in diff])
		if use_particles:
			self.add_particles(context, current_coll, collection_name)

		clean_node_groups(list(diff))

	def add_particles(self, context, collection, coll_name):
		added = context.object.modifiers.new(coll_name, "PARTICLE_SYSTEM")
		parts = bpy.data.particles.new(coll_name)
		parts.type = "HAIR"
		parts.count = context.scene.fade_particles_amount
		parts.render_type = "COLLECTION"
		parts.particle_size = context.scene.fade_particles_size
		parts.instance_collection = collection
		parts.use_rotation_instance = True
		parts.use_collection_pick_random = True
		parts.use_rotations = True
		parts.phase_factor_random = 2
		parts.phase_factor = 0.5
		context.object.particle_systems[added.name].seed = random.randint(
			0, 200)
		context.object.particle_systems[added.name].settings = parts

	def add_fog(self, context, src_obj, objs):
		if src_obj:
			bpy.ops.update.fog()
		else:
			add_fog_to_obj(context, objs)


class ADD_OT_fadeshader(bpy.types.Operator):
	bl_idname = 'add.fadeshader'
	bl_description = 'add shader'
	bl_category = 'Node'
	bl_label = 'Smooth Normals'
	bl_options = {'REGISTER', 'UNDO'}

	@classmethod
	def poll(cls, context):
		if context.scene.fade_choice.lower().startswith("outline"):
			has_outline = [
				o for o in context.collection.objects if "Line Art" in o.name]
			return False if has_outline else True
		else:
			return context.object and context.object.active_material and context.object.active_material.node_tree

	def execute(self, context):
		op_lookup = {
			"Smooth": 0,
			"Vegetation": 2,
			"Toon Solid": 1,
			"Toon Smooth": 1,
			"Ambient Occlusion": 3,
			"Outline": -1,
			"Metal": 3
		}
		prop = context.scene.fade_choice
		op = op_lookup[prop]
		mat_ref = {
			'Smooth': ('Materialexp', 'Smooth Normals'),
			'Vegetation': ('Materialexp', 'Vegetation'),
			'Toon Smooth': ('Materialexp', 'Toon Smooth'),
			'Toon Solid': ('Materialexp', 'Toon Solid'),
			'Ambient Occlusion': ('Materialexp', 'Ambient Occlusion'),
			'Outline': ('Materialexp', 'Outline'),
			'Metal': ('Materialexp', 'Metal')
		}
		nt = context.object.active_material.node_tree
		selected = None
		select_list = [n for n in nt.nodes if n.select == True]
		out = find_output(nt)
		out_src = out.inputs[0].links
		if nt.nodes.active:
			selected = nt.nodes.active
		elif select_list:
			selected = select_list[0]
		elif out_src and out_src[0].from_node:
			selected = out_src[0].from_node
		else:
			selected = out
		if selected:

			select_mat, select_tree = mat_ref[prop]

			if op > 0:
				new_n = self.append_group(context, (select_mat, select_tree))
				if op < 3:
					placeholder = None
					if out_src:
						if out_src[0].from_node.type == "GROUP" and out_src[0].from_node.node_tree.name.startswith("Fog"):
							placeholder = out_src[0].from_node
					if placeholder:
						out.location.x += 200
						placeholder.location.x += 200
						new_n.location.x = placeholder.location.x - 200
						new_n.location.y = placeholder.location.y
						nt.links.new(
							new_n.outputs[0], placeholder.inputs["Base Material"])
					else:

						out.location.x += 200
						new_n.location.x = out.location.x - 200
						new_n.location.y = out.location.y
						nt.links.new(new_n.outputs[0], out.inputs[0])
					if op == 1:
						context.object.active_material.blend_method = "OPAQUE"
						if len(new_n.inputs["Normal"].links) == 0:
							geo = nt.nodes.new(type="ShaderNodeNewGeometry")
							geo.location.x = new_n.location.x
							geo.location.y = new_n.location.y - 300
							out.location.x += 200
							new_n.location.x += 200
							nt.links.new(
								new_n.inputs["Normal"], geo.outputs["Normal"])
					else:
						context.object.active_material.blend_method = "CLIP"
				else:
					new_n.location = selected.location
					new_n.location.y -= selected.height + 50
			elif op == -1:
				current_obj = context.object
				if "Line Art" not in context.collection.objects:
					bpy.ops.object.gpencil_add(type="LINEART_COLLECTION" if bpy.app.version >= (4,2,0) else "LRT_COLLECTION")
					context.active_object.select_set(False)
					context.view_layer.objects.active = current_obj
					context.active_object.select_set(True)

			else:
				if not self.check_node_exists(nt.nodes, select_tree):
					new_n = self.append_group(
						context, (select_mat, select_tree))
					timeout = 200
					to_explore = [selected]
					found_link = False
					while to_explore and timeout > 0:
						new = to_explore.pop()
						if "Normal" in new.inputs:
							new_n.location = (
								new.location.x - 200, new.location.y - 200)
							nt.links.new(
								new_n.outputs[0], new.inputs["Normal"])
							found_link = True
							break
						else:
							for inputs in new.inputs:
								for l in inputs.links:
									to_explore.append(l.from_node)
						timeout -= 1
					if not found_link:
						new_n.location = (
							selected.location.x - 200, selected.location.y - 200)
			if (context.scene.fade_offset_object):
				target_obj = context.object
				target_location = target_obj.location
				ref_obj = bpy.data.objects.new(
					prop + " Reference", None)

				# due to the new mechanism of "collection"
				bpy.context.scene.collection.objects.link(
					ref_obj)

				# empty_draw was replaced by empty_display
				ref_obj.empty_display_size = 2
				ref_obj.empty_display_type = 'SPHERE'

				ref_obj.location = target_location

				coord = nt.nodes.new(type="ShaderNodeTexCoord")
				coord.location.x = new_n.location.x - 200
				coord.location.y = new_n.location.y - 200
				coord.object = ref_obj
				nt.links.new(
					coord.outputs["Object"], new_n.inputs["Offset"])
				ref_obj.parent = target_obj

		return {'FINISHED'}

	def append_group(self, context, keys):
		_, select_group = keys
		exists = self.check_group_exists(select_group)
		if exists == None:
			file_loc = os.path.join(os.path.dirname(
				__file__), "data", "data.blend")
			with bpy.data.libraries.load(file_loc) as (data_from, data_to):
				data_to.node_groups = [
					data_from.node_groups[data_from.node_groups.index(select_group)]]
			target_tree = bpy.data.node_groups[select_group]
		else:
			target_tree = exists
		current_tree = context.object.active_material.node_tree
		current_nodes = current_tree.nodes
		new_n = current_nodes.new(type="ShaderNodeGroup")
		new_n.node_tree = target_tree
		return new_n

	def check_group_exists(self, name):
		for ng in bpy.data.node_groups:
			if ng.name.startswith(name):
				return ng
		return None

	def check_node_exists(self, nodes, name):
		for n in nodes:
			if n.type == "GROUP" and n.node_tree.name.lower().startswith(name):
				return True
		return False


def clean_node_groups(new_objs):
	materials = set()
	for obj in new_objs:
		for slot in bpy.data.objects[obj].material_slots:
			materials.add(slot.material)

	for m in materials:
		if m.node_tree:
			for n in m.node_tree.nodes:
				if n.type == "GROUP":
					name = n.node_tree.name.split(".")[0]
					if name in bpy.data.node_groups:
						n.node_tree = bpy.data.node_groups[name]


def find_output(tree):
	for n in tree.nodes:
		if n.type == "OUTPUT_MATERIAL":
			return n
	return None


class FOG_PT_panel(bpy.types.Panel):
	bl_space_type = 'VIEW_3D'
	bl_region_type = 'UI'
	bl_category = 'Fade'
	bl_label = 'Fog Effect'

	def draw(self, context):
		layout = self.layout
		row = layout.row()
		engine_name = context.scene.render.engine.lower()
		if not("eevee" in engine_name or "cycles" in engine_name):
			row.label(text="Only Cycles / Eevee supported")
		else:
			if not context.object:
				row.label(text="Please select an object")
			elif not context.object.active_material:
				row.label(text="Please add a material")
			elif not context.object.active_material.node_tree:
				row.label(text="Please enable nodes")
			else:
				if self.should_show_settings(context):
					col = row.column(align=True)
					col.label(text="Copy Settings From:")
					col.prop_search(context.scene, "fade_fog_obj",
									context.scene, "objects", text="")
					row = layout.row(align=True)
					if context.scene.fade_fog_obj:
						if context.selected_objects and not (len(context.selected_objects) == 1 and context.selected_objects[0] == context.scene.fade_fog_obj):
							row.operator(
								UPDATE_OT_fog.bl_idname, text="To Selected").update_type = "selected"
						row.operator(UPDATE_OT_fog.bl_idname,
									 text="To Entire Scene").update_type = "scene"
				else:
					row.operator(ADD_OT_fog.bl_idname)

	def should_show_settings(self, context):
		return context.object and context.object.active_material and context.object.active_material.node_tree and check_fog_exists(context.object.active_material.node_tree.nodes)


class FADE_PT_panel(bpy.types.Panel):
	bl_space_type = 'VIEW_3D'
	bl_region_type = 'UI'
	bl_category = 'Fade'
	bl_label = 'Fade Assets'

	def should_display_style(self, context):
		return not (context.scene.fade_category in categories["Constants"][context.scene.fade_type])

	def find_line_art(self, context):
		for o in context.collection.objects:
			if o.grease_pencil_modifiers:
				for m in o.grease_pencil_modifiers:
					if m.type == "GP_LINEART":
						return m

	def display_particles(self, context):
		return context.object and context.scene.fade_category in ["Weather", "Clouds", "Bushes", "Foliage", "Stones", "Grass"] and "palm" not in context.scene.fade_choice.lower()

	def draw(self, context):
		layout = self.layout
		row = layout.row()
		engine_name = context.scene.render.engine.lower()
		if not("eevee" in engine_name or "cycles" in engine_name):
			row.label(text="Only Cycles / Eevee supported")
		else:
			row.prop(context.scene, "fade_type", expand=True)
			row = layout.row(align=True)
			if self.should_display_style(context):
				row.prop(context.scene, "fade_style_type", expand=True)
			if context.scene.fade_type == "Shaders":
				row = layout.row(align=True)
				if not context.object:
					row.label(text="Please select an object", icon="ERROR")
				elif not context.object.active_material:
					row.label(text="Please add a material", icon="ERROR")
				elif not context.object.active_material.node_tree:
					row.label(text="Please enable nodes", icon="ERROR")
				else:
					row.template_icon_view(
						context.scene, "fade_choice", show_labels=True)
					lowered = context.scene.fade_choice.lower()
					if not "solid" in lowered and not "outline" in lowered and not "metal" in lowered:
						row = layout.row()
						row.prop(context.scene, "fade_offset_object")
					elif "outline" in lowered:
						lineart = self.find_line_art(context)
						if lineart:
							col = layout.column(align=True)
							col.prop(
								lineart.target_material.grease_pencil, "color")
							col.prop(lineart, "thickness")
							row = layout.row()
							row.prop(lineart, "use_crease_on_smooth")
					row = layout.row(align=True)
					row.scale_y = 1.5
					row.operator(ADD_OT_fadeshader.bl_idname,
								 text="Add "+context.scene.fade_choice)

			elif context.scene.fade_type == "Models":
				row = layout.row()
				row.prop(context.scene, "fade_category", text="")
				row = layout.row()
				row.template_icon_view(
					context.scene, "fade_choice", show_labels=True)
				row = layout.row()
				box = row.box()
				row = box.row()
				row.prop(context.scene, "fade_fog")
				if context.scene.fade_fog:
					row = box.row()
					col = row.column(align=True)
					col.label(text="Copy Settings From:")
					col.prop_search(context.scene, "fade_fog_obj",
									context.scene, "objects", text="")
				if context.object and self.display_particles(context):
					row = layout.row()
					box = row.box()
					row = box.row()
					row.prop(context.scene, "fade_particles")
					if context.scene.fade_particles:
						row = box.row()
						col = row.column(align=True)
						col.prop(context.scene, "fade_particles_amount")
						col.prop(context.scene, "fade_particles_size")
						row = box.row()
						row.prop(context.scene, "fade_particles_assetoffset")
				row = layout.row()
				row.prop(context.scene, "fade_scale")

				row = layout.row()
				row.scale_y = 1.5
				row.operator(ADD_OT_fademodel.bl_idname,
							 text="Add "+context.scene.fade_choice)
			else:
				row = layout.row(align=True)
				if not context.scene:
					row.label(text="Please add Scene")
				else:
					row.template_icon_view(
						context.scene, "fade_choice", show_labels=True)
					row = layout.row()
					row.prop(context.scene, "fade_sky_shadeless")
					row = layout.row(align=True)
					row.scale_y = 1.5
					row.operator(ADD_OT_fadesky.bl_idname,
								 text="Add "+context.scene.fade_choice)


classes = (ADD_OT_fog, FOG_PT_panel, UPDATE_OT_fog, ADD_OT_fadeshader,
		   ADD_OT_fademodel, ADD_OT_fadesky, FADE_PT_panel)

previews = {}
categories = {
	"Models": {
		"Smooth": {
			"Bushes": ["Bush Ball", "Bush Small", "Bush Medium", "Bush Large", "Bush Detailed"],
			"Clouds": ["Cloud Ball", "Cloud Small", "Cloud Normal", "Cloud Flat", "Cloud Tall"],
			"Foliage": ["Ferns", "Palm Foliage", "Palm Sapling"],
			"Grass": ["Grass Normal", "Grass Small", "Grass Medium", "Grass Large"],
			"Stones": ["Formations", "Stones 1", "Stones 2", "Stones 3"],
			"Trees": ["Pine", "Oak", "Birch", "Maple", "Palm", "Pine Small"],
			"Weather": ["Rain", "Snow", "Fire"],
		},
		"Solid": {
			"Bushes": ["Bush Ball Toon", "Bush Medium Toon", "Bush Large Toon"],
			"Clouds": ["Cloud Ball Toon", "Cloud Small Toon", "Cloud Normal Toon", "Cloud Flat Toon", "Cloud Tall Toon"],
			"Foliage": ["Ferns Toon", "Palm Foliage Toon", "Palm Sapling Toon"],
			"Grass": ["Grass Normal", "Grass Small", "Grass Medium", "Grass Large"],
			"Stones": ["Formations Toon", "Stones 1 Toon", "Stones 2 Toon", "Stones 3 Toon"],
			"Trees": ["Pine Toon", "Oak Toon", "Birch Toon", "Maple Toon", "Palm Toon", "Pine Small Toon"],
			"Weather": ["Rain", "Snow", "Fire"],
		}
	},
	"Skies": {
		"Smooth": {
			"Skies": ["Sky Day", "Sky Night", "Gradient", "Gradient Spherical"],
		},
		"Solid": {
			"Skies": ["Sky Day", "Sky Night", "Gradient", "Gradient Spherical"],
		}
	},
	"Shaders": {
		"Smooth": {
			"Shading": ["Toon Smooth", "Toon Solid", "Outline", "Metal", "Vegetation", "Ambient Occlusion", "Smooth"]
		},
		"Solid": {
			"Shading": ["Toon Smooth", "Toon Solid", "Outline", "Metal", "Vegetation", "Ambient Occlusion", "Smooth"]
		}
	},
	"Constants": {
		"Models": ["Grass", "Weather"],
		"Skies": ["Skies"],
		"Shaders": ["Shading"]
	}
}


def get_items(self, context):
	items = []
	if context.scene.fade_type in categories:
		if context.scene.fade_category in categories[context.scene.fade_type][context.scene.fade_style_type]:
			asset_list = categories[context.scene.fade_type][context.scene.fade_style_type][context.scene.fade_category]
			for c in range(len(asset_list)):
				items.append((asset_list[c], asset_list[c], asset_list[c],
							  previews["main"][asset_list[c]].icon_id, c))
	return items


def get_item_keys(self, context):
	items = []
	if context.scene.fade_type in categories:
		asset_list = list(
			categories[context.scene.fade_type][context.scene.fade_style_type].keys())
		for c in range(len(asset_list)):
			items.append((asset_list[c], asset_list[c], asset_list[c]))
	return items


def category_style_change(self, context):
	context.scene.fade_choice = categories[context.scene.fade_type][
		context.scene.fade_style_type][context.scene.fade_category][0]


def type_change(self, context):
	context.scene.fade_category = list(
		categories[context.scene.fade_type][context.scene.fade_style_type].keys())[0]
	category_style_change(self, context)


def filter_fog_objs(self, fogobj):
	am = fogobj.active_material
	return am and am.node_tree and check_fog_exists(am.node_tree.nodes)


def check_fog_exists(nodes):
	found = False
	for n in nodes:
		if n.type == "GROUP" and n.node_tree.name.startswith("Fog"):
			found = True
			break
	return found


def register():
	import bpy.utils.previews
	pcoll = bpy.utils.previews.new()
	icons_dir = os.path.join(os.path.dirname(__file__), "icons")
	icon_data = [
		("Rain", "rain.png"),
		("Snow", "snow.png"),
		("Pine", "pine.png"),
		("Pine Small", "pine_small.png"),
		("Oak", "oak.png"),
		("Smooth", "smooth_normals_new.png"),
		("Birch", "birch.png"),
		("Maple", "Maple.png"),
		("Palm", "Palm.png"),
		("Ferns", "ferns.png"),
		("Palm Foliage", "palm_foliage.png"),
		("Palm Sapling", "palm_sapling.png"),
		("Formations", "formations.png"),
		("Stones 1", "stones 1.png"),
		("Stones 2", "stones 2.png"),
		("Stones 3", "stones 3.png"),
		("Toon Smooth", "toon_new.png"),
		("Toon Solid", "toon_solid.png"),
		("Sky Day", "sky.png"),
		("Sky Night", "night.png"),
		("Vegetation", "vegetation_new.png"),
		("Grass Normal", "Grass 1.png"),
		("Grass Small", "Grass 2.png"),
		("Grass Medium", "Grass 3.png"),
		("Grass Large", "Grass 4.png"),
		("Cloud Ball", "cloud_ball.png"),
		("Cloud Small", "cloud_small.png"),
		("Cloud Normal", "cloud_normal.png"),
		("Cloud Flat", "cloud_flat.png"),
		("Cloud Tall", "cloud_tall.png"),
		("Bush Ball", "Bush Ball.png"),
		("Bush Small", "Bush Small.png"),
		("Bush Medium", "Bush Medium.png"),
		("Bush Large", "Bush Big.png"),
		("Bush Detailed", "Bush Detailed.png"),
		("Bush Ball Toon", "bush_ball_toon.png"),
		("Bush Medium Toon", "bush_medium_toon.png"),
		("Bush Large Toon", "bush_large_toon.png"),
		("Cloud Ball Toon", "cloud_ball_toon.png"),
		("Cloud Small Toon", "cloud_small_toon.png"),
		("Cloud Normal Toon", "cloud_normal_toon.png"),
		("Cloud Flat Toon", "cloud_flat_toon.png"),
		("Cloud Tall Toon", "cloud_tall_toon.png"),
		("Palm Toon", "palm_toon.png"),
		("Ferns Toon", "ferns_toon.png"),
		("Pine Toon", "pines_toon.png"),
		("Birch Toon", "birch_toon.png"),
		("Maple Toon", "maple_toon.png"),
		("Oak Toon", "oak_toon.png"),
		("Palm Sapling Toon", "palm_sapling_toon.png"),
		("Palm Foliage Toon", "palm_foliage_toon.png"),
		("Formations Toon", "formations_toon.png"),
		("Stones 1 Toon", "stones1_toon.png"),
		("Stones 2 Toon", "stones2_toon.png"),
		("Stones 3 Toon", "stones3_toon.png"),
		("Pine Small Toon", "pine_small_toon.png"),
		("Gradient", "gradient.png"),
		("Gradient Spherical", "gradient_spherical.png"),
		("Ambient Occlusion", "ambient_occlusion.png"),
		("Outline", "outline.png"),
		("Fire", "fire.png"),
		("Metal", "metal.png")
	]
	for k, i in icon_data:
		pcoll.load(k, os.path.join(icons_dir, i), "IMAGE")
	previews["main"] = pcoll
	bpy.types.Scene.fade_choice = bpy.props.EnumProperty(
		name="Select", items=get_items, default=None)
	bpy.types.Scene.fade_category = bpy.props.EnumProperty(
		name="category", items=get_item_keys, default=None, update=category_style_change)
	bpy.types.Scene.fade_type = bpy.props.EnumProperty(name="type", items=[
		("Models", "Models", "Models"),
		("Skies", "Skies", "Skies"),
		("Shaders", "Shaders", "Shaders")
	], update=type_change)
	bpy.types.Scene.fade_scale = bpy.props.FloatProperty(
		name="Spawn Scale", default=1.0)
	bpy.types.Scene.fade_fog = bpy.props.BoolProperty(
		name="Add Fog", default=False)
	bpy.types.Scene.fade_fog_obj = bpy.props.PointerProperty(
		type=bpy.types.Object,
		poll=filter_fog_objs
	)
	bpy.types.Scene.fade_particles = bpy.props.BoolProperty(
		name="Use as Particles", default=False)
	bpy.types.Scene.fade_particles_amount = bpy.props.IntProperty(
		name="Particle Amount", default=100)
	bpy.types.Scene.fade_particles_size = bpy.props.FloatProperty(
		name="Particle Size", default=0.20)
	bpy.types.Scene.fade_particles_assetoffset = bpy.props.FloatProperty(
		name="Asset Offset", description="Where the source asset(s) should be placed", default=-50)

	bpy.types.Scene.fade_style_type = bpy.props.EnumProperty(name="style", items=[
		("Smooth", "Smooth", "Smooth"),
		("Solid", "Solid", "Solid")
	], default=None, update=category_style_change)

	bpy.types.Scene.fade_sky_shadeless = bpy.props.BoolProperty(
		name="Shadeless", default=False)

	bpy.types.Scene.fade_offset_object = bpy.props.BoolProperty(
		name="Add Offset Object", default=False)

	from bpy.utils import register_class
	for cls in classes:
		register_class(cls)
	translate.register()


def unregister():
	for pcoll in previews.values():
		bpy.utils.previews.remove(pcoll)
	previews.clear()
	from bpy.utils import unregister_class
	for cls in classes:
		unregister_class(cls)

	del bpy.types.Scene.fade_choice
	del bpy.types.Scene.fade_category
	del bpy.types.Scene.fade_type
	del bpy.types.Scene.fade_scale
	del bpy.types.Scene.fade_fog
	del bpy.types.Scene.fade_fog_obj
	del bpy.types.Scene.fade_particles
	del bpy.types.Scene.fade_particles_amount
	del bpy.types.Scene.fade_particles_size
	del bpy.types.Scene.fade_particles_assetoffset
	del bpy.types.Scene.fade_style_type
	del bpy.types.Scene.fade_sky_shadeless

	translate.unregister()
